# -*- coding: utf-8 -*-
from importlib.metadata import PackageNotFoundError, version

try:
    # Change here if project is renamed and does not equal the package name
    dist_name = __name__
    __version__ = version(dist_name)
except PackageNotFoundError:
    __version__ = 'unknown'
finally:
    del PackageNotFoundError, version


"""
mafredo = Marine Frequency Domain

"""

from mafredo.helpers import FrequencyUnit, Symmetry, MotionMode
from mafredo.hyddb1 import Hyddb1, Symmetry
from mafredo.rao import Rao

__all__ = ['Hyddb1', 'Symmetry', 'Rao', 'FrequencyUnit', 'Symmetry', 'MotionMode']


from mafredo.helpers import *
from numpy.testing import assert_almost_equal
import numpy as np

def test_wavelength_deep():
    omega = 0.1
    L = wavelength(omega)

    expected = 2*np.pi*9.81 / omega**2

    assert_almost_equal(L, expected)


def test_wavelength_shallow():
    omega = 0.1
    waterdepth = 30
    L = wavelength(omega, waterdepth=waterdepth)

    # omega^2 == g * k * tank(k*h)
    # k = 2*pi / wavelength

    k = 2*np.pi / L
    omega2 = 9.81 * k * np.tanh(k*waterdepth)

    assert_almost_equal(omega**2, omega2)


import numpy as np
import pytest
from numpy.testing import assert_almost_equal, assert_allclose

from mafredo.hyddb1 import Hyddb1

def gimme():
    hyd = Hyddb1.create_from(r'files/barge_100_30_4.dhyd')
    return hyd

def test_check_dimensions():
    hyd = gimme()
    hyd._check_dimensions()


def test_load_dhyd():
    hyd = Hyddb1.create_from(r'files/barge_100_30_4.dhyd')
    hyd.plot()

    print(hyd._mass)

def test_load_nc():
    hyd = Hyddb1.create_from_capytaine(r"files/capytaine.nc")

    omega = 0.01

    mass = hyd.amass(omega=omega)
    damping = hyd.damping(omega=omega)
    force = hyd.force(omega=omega, wave_direction=90)


# def test_save_dhyd_to_no_complex():
#     hyd = Hyddb1.create_from(r'files/barge_100_30_4.dhyd')
#     rao = hyd.force_rao(0)
#     r = rao.to_xarray_nocomplex()
#     print(r)

def test_save_dhyd():
    hyd = Hyddb1.create_from(r'files/barge_100_30_4.dhyd')

    # find a temporary folder to work in
    import tempfile
    from pathlib import Path
    tempdir = Path(tempfile.gettempdir())

    file = tempdir / 'test_write_dhyd_and_read_it_again.dhyd'

    hyd.save_as(file)

    copy = Hyddb1.create_from(file)

    hyd.assert_allclose_to(copy)

    # assert_allclose(hyd._mass.values, copy._mass.values, rtol=1e-3, atol=1)
    # assert_allclose(hyd._damping.values, copy._damping.values, rtol=1e-3, atol=1)
    #
    # for i in range(6):
    #     F0 = hyd._force[i]
    #     Fc = copy._force[i]
    #
    #     assert_allclose(F0._data.amplitude.values, Fc._data.amplitude.values, rtol=1e-3, atol=1)


def test_n_frequencies():
    hyd = gimme()
    assert hyd.n_frequencies == 28

def test_n_frequencies_error():
    """Check that an error is raised if the RAOs have un-equal frequencies when n_frequencies is requested"""
    hyd = Hyddb1()
    hyd._force[1].add_frequency(0.176)
    with pytest.raises(ValueError):
        assert hyd.n_frequencies == 9

def test_n_wave_directions():
    hyd = gimme()
    assert hyd.n_wave_directions == 9

def test_wave_directions():
    hyd = gimme()

    from numpy.testing import assert_almost_equal

    assert_almost_equal(hyd.wave_directions , [  0.  , 22.5 , 45. ,  67.5,  90.,  112.5, 135. , 157.5 ,180. ])

def test_n_wave_directions_error():
    """Check that an error is raised if the RAOs have un-equal directions when n_wave_directions is requested"""
    hyd = Hyddb1()
    hyd._force[1].add_direction(17)
    with pytest.raises(ValueError):
        assert hyd.n_wave_directions == 9

def test_write_hyd_file():
    hyd = gimme()
    hyd.to_hyd_file(r'c:\data\test.hyd', hydrostatics=None)

def test_add_addedmass():
    hyd = Hyddb1()
    hyd.set_amass(omega = 1, m6x6 = np.ones((6,6)))

    # by default omega=0 is set to zeros
    # check if interpolation to omega = 0.2 yields 0.2
    actual02 = hyd.amass(0.2)
    expected = 0.2 * np.ones((6,6))

    assert_almost_equal(actual02, expected)

def test_add_damping():
    hyd = Hyddb1()
    hyd.set_damping(omega = 1, m6x6 = np.ones((6,6)))

    actual03 = hyd.damping(0.3)
    expected = 0.3 * np.ones((6,6))

    assert_almost_equal(actual03, expected)

def test_read_hyd():
    hyd = Hyddb1.create_from_hyd(r'files/barge.hyd')

    hyd.plot(do_show=False)

def test_interpolate_amass():
    hyd = Hyddb1.create_from(r'files/barge_100_30_4.dhyd')
    omegas = np.linspace(0.1, 4, 100)

    hyd.amass(omegas)

def test_interpolate_damping():
    hyd = Hyddb1.create_from(r'files/barge_100_30_4.dhyd')
    omegas = np.linspace(0.1, 4, 100)

    hyd.damping(omegas)

def test_interpolate_damping_single():
    hyd = Hyddb1.create_from(r'files/barge_100_30_4.dhyd')
    omegas = np.linspace(0.1, 4, 100)

    hyd.damping(0.05)


def test_interpolate_forces():
    hyd = Hyddb1.create_from(r'files/barge_100_30_4.dhyd')
    omegas = np.linspace(0.1, 4, 100)

    hyd.force(0.3, wave_direction=90)
    hyd.force(0.301, wave_direction=90)


import numpy as np
import pytest
from numpy.testing import assert_allclose


from mafredo.hyddb1 import Hyddb1



def test_write_hyd_and_read_it_again():
    hyd = Hyddb1.create_from(r'files/barge_100_30_4.dhyd')

    # find a temporary folder to work in
    import tempfile
    from pathlib import Path
    tempdir = Path(tempfile.gettempdir())

    file = tempdir / 'test_write_hyd_and_read_it_again.hyd'

    hyd.to_hyd_file(file, hydrostatics=None)

    copy = Hyddb1.create_from_hyd(file)

    assert_allclose(hyd._mass.values, copy._mass.values, rtol=1e-3, atol = 1 )
    assert_allclose(hyd._damping.values, copy._damping.values, rtol=1e-3, atol = 1 )

    for i in range(6):
        F0 = hyd._force[i]
        Fc = copy._force[i]

        assert_allclose(F0._data.amplitude.values, Fc._data.amplitude.values, rtol=1e-3, atol=1)


    #
    # hyd.plot(phase=False)
    # copy.plot(phase=False)


from mafredo.rao import _complex_unit_add,_complex_unit_delete,_complex_unit_to_phase
import xarray as xr
import numpy as np
from numpy.testing import assert_allclose

def test_raw():
    a = xr.Dataset({
            'amplitude': (['wave_direction', 'omega'], np.zeros((2,2))),
            'phase': (['wave_direction', 'omega'], np.zeros((2,2))),
                    },
            coords={'wave_direction': [0,180],
                    'omega': [0,4],
                    }
        )

    a['phase'].values = [[np.pi, 2*np.pi],
                         [0.5*np.pi,  -0.5*np.pi]]

    _complex_unit_add(a)

    print(a['complex_unit'].values)

    expected = [[-1, 1],
                [1j, -1j]]

    assert_allclose(a['complex_unit'].values, expected)

def test_there_and_back():
    a = xr.Dataset({
        'amplitude': (['wave_direction', 'omega'], np.zeros((2, 2))),
        'phase': (['wave_direction', 'omega'], np.zeros((2, 2))),
    },
        coords={'wave_direction': [0, 180],
                'omega': [0, 4],
                }
    )

    expected = [[np.pi, 2 * np.pi],
                         [0.5 * np.pi, -0.5 * np.pi]]

    a['phase'].values = expected
    _complex_unit_add(a)
    _complex_unit_to_phase(a)

    # to avoid the 2*pi phase jumps, simply use the cos and the sine

    assert_allclose(np.cos(a['phase'].values),
                    np.cos(expected))
    assert_allclose(np.sin(a['phase'].values),
                    np.sin(expected))

def test_cleanup():
    a = xr.Dataset({
        'amplitude': (['wave_direction', 'omega'], np.zeros((2, 2))),
        'phase': (['wave_direction', 'omega'], np.zeros((2, 2))),
    },
        coords={'wave_direction': [0, 180],
                'omega': [0, 4],
                }
    )
    _complex_unit_add(a)
    a = _complex_unit_delete(a)

    print(a.variables)

    assert('complex_unit' not in a.variables)








"""
Merge datasets along frequency
===============================

Example of how to merge refined hydrodynamic databases.

In this example a model was run multiple times with different frequency grids.

RAO data is merged using xarrays 'merge' method while the added-mass and damping are merged
using xarrays 'concat' method along the omega dimension.

"""

from mafredo import *

try:

    # First create the object
    cylinder = Hyddb1.create_from_capytaine('open_cylinder.nc')
    cylinder_refined = Hyddb1.create_from_capytaine('open_cylinder_2.nc')
    cylinder_further_refined = Hyddb1.create_from_capytaine('open_cylinder_3.nc')

    # merge
    cylinder.add(cylinder_refined)
    cylinder.add(cylinder_further_refined)

    # and finally plot
    cylinder.plot()

except:
    pass # read-the-docs


from mafredo import Hyddb1

"""The orcaflex .yml example file contains two vessel types.

Both are 100x30x4 barges, but one is created using double symmetry while the other 
does not employ symmetry at all.

Applying symmetry on the first one should give the same result as the second one.

"""

def test_read_both():

    filename = r'.\files\barge_100x30x4.yml'
    vessel_type_name = 'Vessel type1'
    iDraught = 0

    data = Hyddb1.create_from_orcaflex_yml(filename, vessel_type_name, iDraught)
    data.expand360_using_symmetry()

    filename = r'.\files\barge_100x30x4.yml'
    data2 = Hyddb1.create_from_orcaflex_yml(filename, 'Full_directions')

    data.assert_allclose_to(data2)

def test_convert_orcaflex_to_hyd():
    filename = r'.\files\barge_100x30x4.yml'
    vessel_type_name = 'Vessel type1'
    iDraught = 0

    data = Hyddb1.create_from_orcaflex_yml(filename, vessel_type_name, iDraught)
    data.expand360_using_symmetry()

    import tempfile
    from pathlib import Path
    tempdir = Path(tempfile.gettempdir())

    file = tempdir / 'barge_100x30x4_ofx.hyd'

    data.to_hyd_file(file)

    from os import remove
    remove(file)



from mafredo import hyddb1
import numpy as np
import matplotlib.pyplot as plt


filename = 'cheetah'

hyd = hyddb1.Hyddb1()

print('reading data')

hyd.create_from_capytaine(r"{}.nc".format(filename))

plt.subplots(2,3)
for i in range(6):
    plt.subplot(2,3,i+1)
    rao = hyd.force_rao(i)
    rao['amplitude'].plot()

plt.show()


hyd.save_as("{}.dhyd".format(filename))




from mafredo import hyddb1
import numpy as np
import matplotlib.pyplot as plt


filename = r"c:\data\temp.nc"

hyd = hyddb1.Hyddb1()

print('reading data')

hyd.create_from_capytaine(r"capytaine.nc")
hyd.save_as(filename)

h2 = hyddb1.Hyddb1()
h2.create_from(filename)


"""
Wavelength
===========

Wave-length for a given frequency can be calculated using .. function::helpers.wavelength.

"""

import numpy as np
from mafredo.helpers import wavelength
import matplotlib.pyplot as plt

T = np.linspace(1,20)
omega = 2*np.pi / T

for wd in (5,10,20, 40,100,0):
    wave_lengths = [wavelength(om, waterdepth=wd) for om in omega]
    if wd == 0:
        name = 'deep water'
    else:
        name = f'waterdepth = {wd}m'

    plt.plot(T, wave_lengths, label=name)
plt.legend()
plt.xlabel('Wave period [s]')
plt.ylabel('Wave length [m]')
plt.ylim(0,200)
plt.grid()








"""
Interpolation and symmetry
===========================

Example of extrapolation of values

"""

try:

    from mafredo import *
    import xarray as xr
    import numpy as np
    import matplotlib.pyplot as plt

    # get a RAO
    hyd = Hyddb1.create_from_capytaine(r"capytaine.nc")
    rao = hyd.force_rao(0)

    plt.figure()
    rao['amplitude'].plot()
    plt.title('input')

    # add resolution in omega direction

    omegas = np.linspace(0,4,100)
    for omega in omegas:
        rao.add_frequency(omega)

    # add symmetry
    plt.figure()
    rao['amplitude'].plot()
    plt.title('Re-grid frequencies')

    rao.expand_symmetry_xz()

    plt.figure()
    rao['amplitude'].plot()
    plt.title('Added symmetry')

    # regrid directions
    plt.figure()
    rao.regrid_direction(np.linspace(0,360,80))
    rao['amplitude'].plot()
    plt.title('Regridded directions')

    import matplotlib.pyplot as plt
    # plt.show()
except:
    pass # read-the-docs




from mafredo import *
import xarray as xr
import numpy as np
import matplotlib.pyplot as plt

def test_example():
    # get a RAO
    hyd = Hyddb1.create_from_capytaine(r"./files/capytaine.nc")
    rao = hyd.force_rao(0)

    plt.figure()
    rao['amplitude'].plot()
    plt.title('input')

    # add resolution in omega direction

    rao.add_frequency(0)
    rao.add_frequency(4)
    rao.add_frequency([1,2,1.7,0.6])

    # add symmetry
    plt.figure()
    rao['amplitude'].plot()
    plt.title('Added more frequencies')


    rao.expand_symmetry_xz()

    plt.figure()
    rao['amplitude'].plot()
    plt.title('Added symmetry')


    # regrid directions
    plt.figure()
    rao.regrid_direction(np.linspace(0,360,80))
    rao['amplitude'].plot()
    plt.title('Regridded directions')

    plt.figure()
    rao['phase'].plot()
    plt.title('Regridded directions')

    # plt.show()




import xarray as xr
import numpy as np
from mafredo.rao import Rao
from mafredo.helpers import MotionMode
from mafredo.hyddb1 import Hyddb1
from numpy.testing import assert_allclose

def test_extrapolate():

    test = Rao.create_from_capytaine_wave_force(r"files/capytaine.nc", MotionMode.ROLL)

    test.expand_symmetry_xz()
    test.regrid_omega(np.linspace(0,4,100))
    test.regrid_direction(np.linspace(0, 360, 5))

    value = test.get_value(omega = 0, wave_direction=359)

    assert not np.isnan(value)


def test_extrapolate_on_addedmass():
    hyd = Hyddb1.create_from_capytaine(r"files/capytaine.nc")
    # hyd.add_frequencies([0])

    assert not np.any(np.isnan(hyd.amass(0)))


def test_extrapolate_on_damping():
    hyd = Hyddb1.create_from_capytaine(r"files/capytaine.nc")
    # hyd.add_frequency(0)
    assert not np.any(np.isnan(hyd.damping(0)))

def test_extrapolate_on_force():
    hyd = Hyddb1.create_from_capytaine(r"files/capytaine.nc")
    hyd.add_frequency(0)
    assert not np.any(np.isnan(hyd.force(0,90)))

def test_extrapolate_on_force_dir():
    hyd = Hyddb1.create_from_capytaine(r"files/capytaine.nc")
    hyd.add_direction(17)
    assert not np.any(np.isnan(hyd.force(1,17)))  # automatically added

def test_extrapolate_below_heading_range():
    rao = Rao()
    rao._data = xr.Dataset({
            'amplitude': (['wave_direction', 'omega'], [[10.,10],[0,0]]),
            'phase': (['wave_direction', 'omega'], np.zeros((2,2),dtype=float)),
                    },
            coords={'wave_direction': [10,350.],
                    'omega': [0,4.],
                    }
        )

    rao._data

    rao.add_direction(0)

    rao._data.sel(omega=4)

    val = rao.get_value(0, 0)  # expect 5.0

    assert_allclose(val, 5.0)





